/*
	Intel HEXファイルデコーダー コンストラクタ
*/
HexDecoder = function(buffer) {
	this.size = null;
	this.top = null;
	this.buffer = buffer;
	this._hexDecoderHttp = new XMLHttpRequest();
};

/*
	Index HEXファイルをメモリに書き込む
*/
HexDecoder.prototype._decode = function(hex, off) {
	var top = 0xffff;
	var bottom = 0;
	var r = 0;
	var sum;
	var len;
	var address_h;
	var address_l;
	var address;
	var i;
	var val;

	if(off === undefined)
		off = 0;

	for(;;) {
		/* レコード先頭文字を得る */
		if(hex.charAt(r) != ":")
			throw new Error("ファイルの形式が不正です");
		r++;

		/* データ長を得る */
		sum = len = parseInt(hex.substr(r, 2), 16); r += 2;
		if(len == 0)
			return;

		/* オフセットレコードを得る */
		address_h = parseInt(hex.substr(r, 2), 16); r += 2;
		sum += address_h;
		address_l = parseInt(hex.substr(r, 2), 16); r += 2;
		sum += address_l;
		address = address_h << 8 | address_l;

		/* レコードタイプを得る */
		switch(parseInt(hex.substr(r, 2), 16)) {
		case 0x00: /* データレコード */
			break;
		case 0x01: /* エンドレコード */
			return;
		default: /* その他(未対応) */
			throw new Error("未対応の形式です");
		}
		r += 2;

		/* データを得る */
		for(i = address + off; i < address + off + len; i++) {
			val = parseInt(hex.substr(r, 2), 16); r += 2;
			sum += val;

			if(this.buffer != null)
				this.buffer[i] = val;
		}

		/* サムをチェックする */
		sum = (~sum + 1) & 0xff;
		/*
		if(sum != parseInt(hex.substr(r, 2), 16))
			throw new Error("チェックサムが合いません");
		*/
		r += 2;

		/* 先頭とサイズを得る */
		if(top > address)
			top = address;
		if(bottom < address)
			bottom = address;
		this.top = top;
		this.size = bottom - top;

		/* 次の行まで移動する */
		while(r < hex.length && (hex.charAt(r) == "\r" || hex.charAt(r) == "\n"))
			r++;
	}
};

/*
	Index HEXファイルの先頭とファイルサイズを得る
*/
HexDecoder.prototype.check = function(hex) {
	var tmp = this.buffer;
	this._decode(hex);
	this.buffer = tmp;
};

/*
	Index HEXファイルをメモリに書き込む
*/
HexDecoder.prototype.decode = function(hex, address) {
	if(address === undefined)
		this._decode(hex);
	else {
		this.check(hex);
		this._decode(hex, address - this.top);
	}
};

/*
	Index HEXファイルをサーバから読み込む
*/
HexDecoder.prototype.read = function(url, address) {
	this._hexDecoderHttp.open("GET", url, false);
	this._hexDecoderHttp.send(null);
	this.decode(this._hexDecoderHttp.responseText, address);
};
